import { createJWKS, createKeyPair, signJwt } from './tools.js';
import { setupServer } from 'msw/node';
import { HttpResponse, http } from 'msw';
export const createJWKSMock = (jwksBase, jwksPath = '/.well-known/jwks.json') => {
    const keypair = createKeyPair();
    const JWKS = createJWKS({
        ...keypair,
        jwksOrigin: jwksBase,
    });
    const handler = http.get(new URL(jwksPath, jwksBase).href, () => HttpResponse.json(JWKS));
    const kid = () => JWKS.keys[0].kid;
    let server;
    const start = () => {
        server = setupServer(handler);
        server.listen({ onUnhandledRequest: 'bypass' });
        return () => server.close();
    };
    const stop = () => {
        server.close();
    };
    const token = (token = {}) => signJwt(keypair.privateKey, token, kid());
    return {
        start,
        /**
         * @deprecated Use the thunk returned by `start` instead.
         */
        stop,
        kid,
        token,
        mswHandler: handler,
    };
};
/**
 * @deprecated Use the named export instead
 */
export default createJWKSMock;
